module(...,package.seeall)

require"socket"
require"nvm"
require"utils"

--调度中心服务器信息
local ddzxIp,ddzxTcpPort = nvm.get("ddzxIp"),nvm.get("ddzxTcpPort")
local ddzxWaitRecvMsgMaxTime = nvm.get("ddzxWaitRecvMsgMaxTime")
local devId = nvm.get("devId")
local devSn = nvm.get("devSn")

--ntp同步时间结果
local timerSyncResult = false

--长连接socket连接状态
local longLinkSocketConnState = false

--前置机服务器信息
local qzjIp,qzjTcpPort
local qzjFrameSn = 0x0000

--协议参数
local prefix = 0xdc
local version = 0x0000
local encryptType = 0x00
local channel = 0x00
local errorCode = 0x0000

--待上报消息队列最大长度
local MSG_QUENE_MAX_LENGTH = nvm.get("msgQueueMaxLen") or 3000


--十六进制字符串反转
local function hexStrReverse(hexstr)
    local str = ""
    for i = 1, string.len(hexstr) - 1, 2 do  
        local tmpStr = string.sub(hexstr, -(i+1), -i)
        str = str .. tmpStr
    end 
    return str
end
--openMulByteReverse:开启多字节反转
local function hex2bin( hexstr, openMulByteReverse )
    --检查字符串长度
	if(hexstr:len()%2~=0) then
	    return false,"hex2str invalid input lenth"
	end
    if openMulByteReverse then
        hexstr = hexStrReverse(hexstr)
    end 
    local str = ""
    for i = 1, string.len(hexstr) - 1, 2 do  
    	local doublebytestr = string.sub(hexstr, i, i+1);  
    	local n = tonumber(doublebytestr, 16);  
    	if 0 == n then  
        	str = str .. '\00'
    	else  
        	str = str .. string.format("%c", n)
    	end
    end 
    return str
end

local function hex2str(str)
	--判断输入类型	
	if (type(str)~="string") then
	    return nil,"hex2str invalid input type"
	end
	--滤掉分隔符
	str=str:gsub("[%s%p]",""):upper()
	--检查内容是否合法
	if(str:find("[^0-9A-Fa-f]")~=nil) then
	    return nil,"hex2str invalid input content"
	end
	--检查字符串长度
	if(str:len()%2~=0) then
	    return nil,"hex2str invalid input lenth"
	end
	--拼接字符串
	local index=1
	local ret=""
	for index=1,str:len(),2 do
	    ret=ret..string.char(tonumber(str:sub(index,index+1),16))
	end
    --log.info("socketHeartbeatTask.hex2str", ret)
	return ret
end

local function getFormatTimeBin(time)
    local tClock = time and os.date("*t",time) or os.date("*t")
    local formatTimeBin = string.char(tonumber((""..tClock.year):sub(3,4)))
                            ..string.char(tClock.month)
                            ..string.char(tClock.day)
                            ..string.char(tClock.hour)
                            ..string.char(tClock.min)
                            ..string.char(tClock.sec)
    return formatTimeBin
end

--计算头部校验码
local function getHeadVerifyCode(packet) 
    if not packet then
        log.info("longLinkSocketTask.getHeadVerifyCode", "packet empty!!")
        return false
    end
    if #packet <22 then
        log.info("longLinkSocketTask.getHeadVerifyCode", "packet length < 22!! ",packet:toHex())
        return false
    end
    local CRC = 0XFFFF
    for i=1,22 do
        CRC = bit.bxor( CRC, string.byte(packet:sub(i,i)))
    end
    local verifyCodeBin = string.char(bit.band(CRC,0x00FF))

    return true, verifyCodeBin
end

--计算尾部校验码
local function getTailVerifyCode(packet) 
    if not packet then
        log.info("longLinkSocketTask.getTailVerifyCode", "packet empty!!")
        return false
    end
    
    local CRC = 0XFFFF
    if #packet <= 0 then
        CRC = 0X0000
    else
        for i=1,#packet do
            CRC = bit.bxor( CRC, string.byte(packet:sub(i,i)))
            for j=1,8 do
                if bit.band(CRC,1) ~= 0 then
                    CRC = bit.bxor(bit.rshift(CRC,1), 0xA001)
                else
                    CRC = bit.rshift(CRC,1)
                end
            end
        end
    end
    local verifyCodeBin = string.char(bit.band(CRC,0x00FF))..string.char(bit.rshift(CRC,8))
    return true, verifyCodeBin
end


--生成报文
local function createPacket(frameSn,messageType,commandId,reserve,content) 
    if not prefix then
        log.info("longLinkSocketTask.createPacket", "prefix empty!!")
        return false
    end
    if not version then
        log.info("longLinkSocketTask.createPacket", "version empty!!")
        return false
    end
    if not encryptType then
        log.info("longLinkSocketTask.createPacket", "encryptType empty!!")
        return false
    end
    if not channel then
        log.info("longLinkSocketTask.createPacket", "channel empty!!")
        return false
    end
    if not errorCode then
        log.info("longLinkSocketTask.createPacket", "errorCode empty!!")
        return false
    end
    if not devSn then
        log.info("longLinkSocketTask.createPacket", "devSn empty!!")
        return false
    end
    if not devId then
        log.info("longLinkSocketTask.createPacket", "devId empty!!")
        return false
    end
    if not frameSn then
        log.info("longLinkSocketTask.createPacket", "frameSn empty!!")
        return false
    end
    if not messageType then
        log.info("longLinkSocketTask.createPacket", "messageType empty!!")
        return false
    end
    if not commandId then
        log.info("longLinkSocketTask.createPacket", "commandId empty!!")
        return false
    end
    if not reserve then
        log.info("longLinkSocketTask.createPacket", "reserve empty!!")
        return false
    end

    local packetBin = ""
    local frameSnHexStr = string.format("%04x",frameSn)
    local devIdHexStr = string.format("%08x", commandId == 0x0002 and 0x00000000 or devId)
    local commandIdHexStr = string.format("%04x",commandId)
    local reserveHexStr = string.format("%08x",reserve)
    local contentSizeHexStr = string.format("%04x",#content)
    local errorCodeHexStr = string.format("%04x",errorCode)
    packetBin = string.char(prefix)..hex2bin(string.format("%04x",version))..string.char(encryptType)..hex2bin(frameSnHexStr,true)..string.char(channel)
                        ..hex2bin(errorCodeHexStr)..hex2bin(devIdHexStr,true)..string.char(messageType)..hex2bin(commandIdHexStr)
                        ..hex2bin(reserveHexStr)..hex2bin(contentSizeHexStr,true)
    
    local headRes, headVerifyCodeBin = getHeadVerifyCode(packetBin)
    if not headRes then
        log.info("longLinkSocketTask.createPacket", "getHeadVerifyCode failed!!")
        return false
    end

    packetBin = packetBin..headVerifyCodeBin..content
    local tailRes, tailVerifyCodeBin = getTailVerifyCode(packetBin)
    if not tailRes then
        log.info("longLinkSocketTask.createPacket", "getTailVerifyCode failed!!")
        return false
    end

    packetBin = packetBin..tailVerifyCodeBin
    --log.info("longLinkSocketTask.createPacket", "packetBin hex:",packetBin:toHex())

    return true, packetBin
end

--解析响应报文
local function parseRespPacket(packet)
    if not packet then
        log.info("longLinkSocketTask.parseRespPacket", "packet empty!!")
        return false
    end
    if #packet < 25 then
        log.info("longLinkSocketTask.parseRespPacket", "packet length < 25!! packet:", packet:toHex())
        return false
    end


    local prefixP = string.byte(packet:sub(1,1))
    if prefixP ~= prefix then
        log.info("longLinkSocketTask.parseRespPacket", "prefix invalid!! prefixP:", prefixP)
        return false
    end

    local messageTypeP = pack.unpack(packet:sub(14),">b")
    if messageTypeP ~= 0x02 then
        log.info("longLinkSocketTask.parseRespPacket", "messageType invalid!! messageTypeP:", messageTypeP)
        return false
    end
    
    local contentStart = 24
    --local contentLenP = pack.unpack(packet:sub(21,22),"<H")
    local contentLenP = string.byte(packet:sub(22,22))*16 + string.byte(packet:sub(21,21))
    local contenP = ""

    local parseRes = {}
    
    local commandIdP = string.byte(packet:sub(15,15))*16 + string.byte(packet:sub(16,16))
    parseRes["commandId"] = commandIdP
    parseRes["reserve"] = packet:sub(17,20):toHex()
    log.info("longLinkSocketTask.parseRespPacket","commandIdP:", packet:sub(15,16):toHex(),"contentLenP:",contentLenP,"messageTypeP:",messageTypeP)

    if commandIdP == 0x0002 then
        if contentLenP == 0 then
            log.info("longLinkSocketTask.parseRespPacket", "contentLen invalid!! ")
            return false
        end
        if contentLenP + 25 ~= #packet then
            log.info("longLinkSocketTask.parseRespPacket", "packet format error!! packet:", packet:toHex())
            return false
        end
        contenP = packet:sub(24,contentLenP+23)
        local splitPos = string.find(contenP,"\/")
        if not splitPos then 
            log.info("longLinkSocketTask.parseRespPacket","split char \/ not found!! contenP:", contenP)
            return false 
        end
        local qzjIpP = string.sub(contenP,1,splitPos-1) 
        local qzjPortP = string.sub(contenP,splitPos+1) 
        parseRes["qzjIp"] = qzjIpP
        parseRes["qzjPortP"] = qzjPortP
        qzjIp = qzjIpP
        qzjTcpPort = qzjPortP
        log.info("longLinkSocketTask.parseRespPacket","parseRes qzjIpP:", qzjIpP, "qzjPortP:", qzjPortP)
        return true, parseRes
    elseif commandIdP == 0x0003 then
        return true, parseRes
    elseif commandIdP == 0x0018 then
        if contentLenP ~= 0x06 then
            log.info("longLinkSocketTask.parseRespPacket", "contentLen invalid!! ")
            return false
        end
        if contentLenP + 25 ~= #packet then
            log.info("longLinkSocketTask.parseRespPacket", "packet format error!! packet:", packet:toHex())
            return false
        end
        contenP = packet:sub(24,contentLenP+23)
        local Y,m,d,h,i,s = string.byte(contenP:sub(1,1))+2000
                            ,string.byte(contenP:sub(2,2))
                            ,string.byte(contenP:sub(3,3))
                            ,string.byte(contenP:sub(4,4))
                            ,string.byte(contenP:sub(5,5))
                            ,string.byte(contenP:sub(6,6))
        local serverTime = os.time({day=d, month=m, year=Y, hour=h, minute=i, second=s})
        parseRes["serverTime"] = serverTime
        log.info("longLinkSocketTask.parseRespPacket","serverTime:", serverTime)
        return true, parseRes
    elseif commandIdP == 0x0007 then
        
        return true, parseRes
    else
        log.info("longLinkSocketTask.parseRespPacket","commandIdP invalid!!")
        return false
    end

end

--数据发送的消息队列
local msgQuene = {}

local function insertMsg(data)
    if #msgQuene >= MSG_QUENE_MAX_LENGTH then
        log.warn("longLinkSocketTask.insertMsg", "------> #msgQuene >= MSG_QUENE_MAX_LENGTH !! skip insert !! <-------", #msgQuene, MSG_QUENE_MAX_LENGTH)
        --table.remove(msgQuene,1)
        return false
    end
    table.insert(msgQuene,data)
    return true
end

function waitForSend()
    return #msgQuene > 0
end

function outMsgprocess(socketClient)
    --队列中有消息
    while #msgQuene>0 do
        --获取消息，并从队列中删除
        local outMsg = table.remove(msgQuene,1)
        log.info("longLinkSocketTask.outMsgprocess","send qzj message:",outMsg:toHex(),os.time())
        --发送这条消息，并获取发送结果
        local result = socketClient:send(outMsg)
        --发送失败的话立刻返回nil（等同于false）
        if not result then return end
    end
    return true
end

function inMsgProcess(socketClient)
    local result,data
    while true do
        result,data = socketClient:recv(2000)
        --接收到数据
        if result then  --接收成功
            log.info("longLinkSocketTask.inMsgProcess","receive qzj message:",data:toHex())
            local res,parseData = parseRespPacket(data)
            if res then
                --处理data数据
                if parseData.commandId == 0x0003 then --收到前置机连接应答报文
                    qzjFrameSn = qzjFrameSn + 1
                    local res, qzjGetTimePacket = createPacket(qzjFrameSn,0x1,0x0018,0x00000000,"")
                    if res then
                        insertMsg(qzjGetTimePacket)
                    else
                        log.error("longLinkSocketTask.inMsgProcess","createPacket qzjGetTimePacket failed!!")
                    end
                elseif parseData.commandId == 0x0018 then --收到前置机获取时间应答报文
                    qzjFrameSn = qzjFrameSn + 1
                    local timeBin = getFormatTimeBin()
                    local content = string.char(0x03)..string.char(0x00)..timeBin
                    local res, qzjReportTimePacket = createPacket(qzjFrameSn,0x1,0x0007,0x05000000,content)
                    if res then
                        insertMsg(qzjReportTimePacket)
                    else
                        log.error("longLinkSocketTask.inMsgProcess","createPacket qzjReportTimePacket failed!! reserve(0x05000000)")
                    end
                elseif parseData.commandId == 0x0007 then
                    if parseData.reserve == "05000000" then
                        local timeBin = getFormatTimeBin()
                        local content = string.char(0x03)..string.char(0x00)..timeBin
                        local res, qzjReportTimePacket = createPacket(0x0000,0x1,0x0007,0x02000000,content)
                        if res then
                            insertMsg(qzjReportTimePacket)
                        else
                            log.error("longLinkSocketTask.inMsgProcess","createPacket qzjReportTimePacket failed!! reserve(0x02000000)")
                        end
                    elseif parseData.reserve == "02000000" then
                        local timeBin = getFormatTimeBin()
                        local content = string.char(0x03)..string.char(0xf0)..timeBin..hex2bin("923c0aaace84c4341432c013e3")
                        qzjFrameSn = qzjFrameSn + 1
                        local res, qzjHeartbeatFirstPacket = createPacket(qzjFrameSn,0x1,0x0007,0x04003c0a,content)
                        if res then
                            insertMsg(qzjHeartbeatFirstPacket)
                        else
                            log.error("longLinkSocketTask.inMsgProcess","createPacket qzjHeartbeatFirstPacket failed!!")
                        end
                        sys.publish("MESSAGE_TYPE_START_HEARTBEAT")
                    end
                else
                    log.error("longLinkSocketTask.inMsgProcess","parseData.commandId invalid!! commandId:", parseData.commandId)
                end
            else
                log.error("longLinkSocketTask.inMsgProcess","parseRespPacket failed!!",data:toHex())
            end
            --如果msgQuene中有等待发送的数据，则立即退出本循环
            if waitForSend() then return true end
        else    --接收失败
            log.info("longLinkSocketTask.inMsgProcess","wait receive qzj message timeout")
            --如果msgQuene中有等待发送的数据，则立即退出本循环
            if waitForSend() then return true end
            break
        end
    end
    --返回结果，处理成功返回true，处理出错返回false
    return result or data=="timeout"
end



--连接前置机主任务
sys.taskInit(
function()
    
    while true do
        log.info("longLinkSocketTask.qzjTask","wait ON_MESSAGE_DDZX_AUTH_SUCCESS message ...")
        local res,data = sys.waitUntil("ON_MESSAGE_DDZX_AUTH_SUCCESS", 3600*1000)
        if res then
            log.info("longLinkSocketTask.qzjTask","receive ON_MESSAGE_DDZX_AUTH_SUCCESS message")
            break
        else
            log.info("longLinkSocketTask.qzjTask","wait ON_MESSAGE_DDZX_AUTH_SUCCESS timeout")
        end
    end

    local retryConnectCnt = 0   --失败次数统计
    while true do
        --是否获取到了分配的ip(是否连上网)
        if socket.isReady() then
            --新建一个socket对象，如果是udp只需要把tcp改成udp即可
            local socketClient = socket.tcp()
            --尝试连接指定服务器
            if socketClient:connect(qzjIp,qzjTcpPort) then
                --连接成功
                log.info("longLinkSocketTask.qzjTask","connect success")
                retryConnectCnt = 0 --失败次数清零
                longLinkSocketConnState = true

                --循环处理接收和发送的数据
                while true do
                    if not inMsgProcess(socketClient) then  --接收消息处理函数
                        log.error("longLinkSocketTask.qzjTask","longLinkSocketTask.inMsgProcess error")
                        break
                    end
                    if not outMsgprocess(socketClient) then --发送消息处理函数
                        log.error("longLinkSocketTask.qzjTask","longLinkSocketTask.outMsgprocess error")
                        break
                    end
                end
            else
                log.info("longLinkSocketTask.qzjTask","connect fail")
                --连接失败
                retryConnectCnt = retryConnectCnt+1 --失败次数加一
            end
            socketClient:close()    --断开socket连接
            if retryConnectCnt>=5 then  --失败次数大于五次了
                link.shut()         --强制断开TCP/UDP连接
                retryConnectCnt=0   --失败次数清零
                longLinkSocketConnState = false

                --停止重连前置机的尝试,重新从调度中心获取可用的端口
                sys.publish("ON_MESSAGE_DO_DDZX_TASK",true)
                while true do
                    log.info("longLinkSocketTask.qzjTask","wait ON_MESSAGE_DDZX_AUTH_SUCCESS message ...")
                    local res,data = sys.waitUntil("ON_MESSAGE_DDZX_AUTH_SUCCESS", 3600*1000)
                    if res then
                        log.info("longLinkSocketTask.qzjTask","receive ON_MESSAGE_DDZX_AUTH_SUCCESS message")
                        break
                    else
                        log.info("longLinkSocketTask.qzjTask","wait ON_MESSAGE_DDZX_AUTH_SUCCESS timeout")
                    end
                end
            else
                sys.wait(5000)
            end
            
        else
            retryConnectCnt = 0     --没连上网，失败次数清零
            longLinkSocketConnState = false
            --没连上网
            --等待网络环境准备就绪，超时时间是5分钟
            sys.waitUntil("IP_READY_IND",300000)
            --等完了还没连上？
            if not socket.isReady() then
                --进入飞行模式，20秒之后，退出飞行模式
                net.switchFly(true)
                sys.wait(20000)
                net.switchFly(false)
            end
        end
    end
end)


--等待时间同步成功消息通知
sys.subscribe("TIMER_SYNC_SUCCESS", function ( ... )
    log.info("longLinkSocketTask.subscribe","TIMER_SYNC_SUCCESS")
    timerSyncResult = true
end)


--连接调度中心(获取前置机连接信息)
sys.taskInit(
function()
    
    while not timerSyncResult do
        log.info("longLinkSocketTask.ddzxSocketClient","wait TIMER_SYNC_SUCCESS message ...")
        sys.wait(1000) 
    end
    while not devSn do 
        log.info("longLinkSocketTask.ddzxSocketClient","devSn empty! time:", os.time())
        devSn = nvm.get("devSn")
        sys.wait(1000) 
    end

    while not devId do 
        log.info("longLinkSocketTask.ddzxSocketClient","devId empty! time:", os.time())
        devId = nvm.get("devId")
        sys.wait(1000) 
    end

    while not ddzxIp do 
        log.info("longLinkSocketTask.ddzxSocketClient","ddzxIp empty! time:", os.time())
        ddzxIp = nvm.get("ddzxIp")
        sys.wait(1000) 
    end

    while not ddzxTcpPort do 
        log.info("longLinkSocketTask.ddzxSocketClient","ddzxTcpPort empty! time:", os.time())
        ddzxTcpPort = nvm.get("ddzxTcpPort")
        sys.wait(1000) 
    end

    while not ddzxWaitRecvMsgMaxTime do 
        log.info("longLinkSocketTask.ddzxSocketClient","ddzxWaitRecvMsgMaxTime empty! time:", os.time())
        ddzxWaitRecvMsgMaxTime = nvm.get("ddzxWaitRecvMsgMaxTime")
        sys.wait(1000) 
    end
    local retryConnectCnt = 0   --失败次数统计
    while true do
        --是否获取到了分配的ip(是否连上网)
        if socket.isReady() then
            --新建一个socket对象，如果是udp只需要把tcp改成udp即可
            local ddzxSocketClient = socket.tcp()
            --尝试连接指定服务器
            if ddzxSocketClient:connect(ddzxIp,ddzxTcpPort) then
                local result,packet_data = createPacket(0,0x1,0x0002,0x00000000,devSn)
                if not result then
                    log.info("longLinkSocketTask.ddzxSocketClient", "createDdzxAuthPacket failed!!")
                else
                    --连接成功
                    log.info("longLinkSocketTask.ddzxSocketClient", "ddzxIp", ddzxIp, "ddzxTcpPort", ddzxTcpPort, "packet_data", packet_data:toHex())
                    if ddzxSocketClient:send(packet_data, 10) then
                        retryConnectCnt = 0
                        log.info("longLinkSocketTask.ddzxSocketClient", "wait recv server message ... time:", os.time())
                        local res,ddzx_data = ddzxSocketClient:recv(ddzxWaitRecvMsgMaxTime*1000)
                        if not res then
                            log.info("longLinkSocketTask.ddzxSocketClient", "ddzxSocketClient:recv(10000) wait timeout, time:", os.time())
                        else
                            --TODO：处理收到的数据data
                            log.info("longLinkSocketTask.ddzxSocketClient", "receive server message, time: ", os.time(), ", message:", ddzx_data:toHex())

                            --解析数据并响应结果
                            local res,parseData = parseRespPacket(ddzx_data)
                            if not res then 
                                log.info("longLinkSocketTask.ddzxSocketClient","parseRespPacket failed!!")
                            else
                                log.info("longLinkSocketTask.ddzxSocketClient","publish ON_MESSAGE_DDZX_AUTH_SUCCESS，task finish,wait ON_MESSAGE_DO_DDZX_TASK ...")
                                sys.publish("ON_MESSAGE_DDZX_AUTH_SUCCESS",parseData)
                                --进入阻塞状态，等待下一次连接调度中心的任务
                                while true do
                                    local result,msg = sys.waitUntil("ON_MESSAGE_DO_DDZX_TASK", 3600*1000)
                                    if result and msg ~="CLOSED" then
                                        log.info("longLinkSocketTask.ddzxSocketClient","receive ON_MESSAGE_DO_DDZX_TASK message, msg:", msg)
                                        break
                                    else
                                        log.info("longLinkSocketTask.ddzxSocketClient","wait ON_MESSAGE_DO_DDZX_TASK timeout")
                                    end
                                end
                            end
                        end
                    else
                        retryConnectCnt = retryConnectCnt+1
                        log.info("longLinkSocketTask.ddzxSocketClient",">>>>>>> socket.send failed")
                        --sys.wait(5000)
                    end
                end

            else
                log.info("longLinkSocketTask.ddzxSocketClient","connect fail")
                --连接失败
                retryConnectCnt = retryConnectCnt+1 --失败次数加一
            end
            ddzxSocketClient:close()    --断开socket连接
            if retryConnectCnt>=5 then  --失败次数大于五次了
                link.shut()         --强制断开TCP/UDP连接
                retryConnectCnt=0   --失败次数清零
            end
            sys.wait(5000)
        else
            retryConnectCnt = 0     --没连上网，失败次数清零
            --没连上网
            --等待网络环境准备就绪，超时时间是5分钟
            sys.waitUntil("IP_READY_IND",300000)
            --等完了还没连上？
            if not socket.isReady() then
                --进入飞行模式，20秒之后，退出飞行模式
                net.switchFly(true)
                sys.wait(20000)
                net.switchFly(false)
            end
        end
    end
end)

sys.subscribe("ON_MESSAGE_DDZX_AUTH_SUCCESS", function()
    qzjFrameSn = 0x0001
    local res, qzjConnPacket = createPacket(qzjFrameSn,0x01,0x0003,0x00000000,"")
    if res then
        insertMsg(qzjConnPacket)
    else
        log.info("longLinkSocketTask.qzjLogin","createPacket failed!!")
    end
    
end)

--设备发送心跳程序
sys.taskInit(
function()
    while true do
        log.info("longLinkSocketTask.qzjTask","wait MESSAGE_TYPE_START_HEARTBEAT message ...")
        local res,data = sys.waitUntil("MESSAGE_TYPE_START_HEARTBEAT", 3600*1000)
        if res then
            log.info("longLinkSocketTask.qzjTask","receive MESSAGE_TYPE_START_HEARTBEAT message")
            break
        else
            log.info("longLinkSocketTask.qzjTask","wait MESSAGE_TYPE_START_HEARTBEAT timeout")
        end
    end
    sys.wait(120000)
    while true do
        if socket.isReady() then    --连上网再开始运行
            local timeBin = getFormatTimeBin()
            local content = string.char(0x05)..string.char(0x00)..timeBin..hex2bin("010048000100010002")
            qzjFrameSn = qzjFrameSn + 1
            local res, qzjHeartbeatPacket = createPacket(qzjFrameSn,0x1,0x0007,0x00000000,content)
            if res then
                insertMsg(qzjHeartbeatPacket)
            else
                log.error("longLinkSocketTask.inMsgProcess","createPacket qzjHeartbeatPacket failed!!")
            end
            sys.wait(120000)         --等待一段时间发下次心跳
        else    --没连上网别忘了延时！不然会陷入while true死循环，导致模块无法运行其他代码
            sys.wait(1000)          --等待1秒
        end
    end
end)

-- 解析并上报rfid数据
sys.taskInit(function()
    
    --等心跳开启时，再开始上报rfid数据
    while true do
        log.info("longLinkSocketTask.parse_packet","wait MESSAGE_TYPE_START_HEARTBEAT message ...")
        local res,data = sys.waitUntil("MESSAGE_TYPE_START_HEARTBEAT", 3600*1000)
        if res then
            log.info("longLinkSocketTask.parse_packet","receive MESSAGE_TYPE_START_HEARTBEAT message")
            break
        else
            log.info("longLinkSocketTask.parse_packet","wait MESSAGE_TYPE_START_HEARTBEAT timeout")
        end
    end
    local rfidFrameSn = 0
    while true do 
        --log.info("socketTask.pub_msg",">>>>>>>>>>> waitUntil pub_packet_data_tiandiren ")
        --阻塞监听监听串口接收数据
        local cur_time = os.time()
        local result, packet_data = sys.waitUntil("pub_packet_data_tiandiren",300*1000)
        if result then
            --log.info("longLinkSocketTask.pub_msg",">>>>>>>>> recv pub_packet_data_tiandiren, times(s):", os.time()-cur_time, json.encode(packet_data))
            
            cur_time = os.time()
            local tmp_bin = ""
            for key, rfid in pairs(packet_data["buf"]) do
                if type(rfid) == "table" then
                    if not rfid.time then
                        log.info("longLinkSocketTask.parse_packet",">>>>>>>>> rfid.time empty! ", type(rfid))
                        for k, v in pairs(rfid) do
                            log.info("longLinkSocketTask.parse_packet",">>>>>>>>> rfid", "k:", k, ", v:", v)
                        end
                    else
                        local timeBin = getFormatTimeBin(rfid.time)
                        local content = string.char(0x03)..string.char(0xf0)..timeBin..hex2bin("008004")..hex2bin(key)..hex2bin("01ffff000100")
                        rfidFrameSn = rfidFrameSn + 1
                        local res, qzjRfidPacket = createPacket(rfidFrameSn,0x1,0x0007,0x07008004,content)
                        if res then
                            insertMsg(qzjRfidPacket)
                        else
                            log.error("longLinkSocketTask.parse_packet","createPacket qzjRfidPacket failed!!")
                        end
                    end 
                    
                else
                    log.info("longLinkSocketTask.parse_packet",">>>>>>>>> rfid error! key = ", key)
                end
            end
            
        else
            log.info("longLinkSocketTask.parse_packet",">>>>>>>>> wait pub_packet_data_tiandiren timeout")
            --sys.wait(1000)
        end
    end
end)


-- 监听并上报rfid数据
sys.taskInit(function()
    
    --等心跳开启时，再开始上报rfid数据
    while true do
        log.info("longLinkSocketTask.parse_packet","wait MESSAGE_TYPE_START_HEARTBEAT message ...")
        local res,data = sys.waitUntil("MESSAGE_TYPE_START_HEARTBEAT", 3600*1000)
        if res then
            log.info("longLinkSocketTask.parse_packet","receive MESSAGE_TYPE_START_HEARTBEAT message")
            break
        else
            log.info("longLinkSocketTask.parse_packet","wait MESSAGE_TYPE_START_HEARTBEAT timeout")
        end
    end
    local rfidFrameSn = 0
    while true do 
        --log.info("socketTask.pub_msg",">>>>>>>>>>> waitUntil pub_packet_data_tiandiren_rfid_info ")
        --阻塞监听监听rfid接收数据
        local cur_time = os.time()
        local result, rfid = sys.waitUntil("pub_packet_data_tiandiren_rfid_info",300*1000)
        if result then
            --log.info("longLinkSocketTask.upload_tiandiren_rfid_info",">>>>>>>>> recv pub_packet_data_tiandiren_rfid_info, times(s):", os.time()-cur_time, json.encode(rfid))
            
            if type(rfid) == "table" then
                if not rfid.time then
                    log.info("longLinkSocketTask.upload_tiandiren_rfid_info",">>>>>>>>> rfid.time empty! ")
                    for k, v in pairs(rfid) do
                        log.info("longLinkSocketTask.upload_tiandiren_rfid_info",">>>>>>>>> rfid", "k:", k, ", v:", v)
                    end
                elseif not rfid.id then
                    log.info("longLinkSocketTask.upload_tiandiren_rfid_info",">>>>>>>>> rfid.id empty! ")
                else
                    log.info("longLinkSocketTask.upload_tiandiren_rfid_info","rfid.id:",rfid.id,",rfid.time:",rfid.time,",rfid.id_dec:",tonumber(rfid.id, 16))
                    local timeBin = getFormatTimeBin(rfid.time)
                    local content = string.char(0x03)..string.char(0xf0)..timeBin..hex2bin("008004")..hex2bin(rfid.id)..hex2bin("01ffff000100")
                    rfidFrameSn = rfidFrameSn + 1
                    local res, qzjRfidPacket = createPacket(rfidFrameSn,0x1,0x0007,0x07008004,content)
                    if res then
                        insertMsg(qzjRfidPacket)
                    else
                        log.error("longLinkSocketTask.upload_tiandiren_rfid_info","createPacket qzjRfidPacket failed!!")
                    end
                end 
                
            else
                log.info("longLinkSocketTask.upload_tiandiren_rfid_info",">>>>>>>>> rfid error! key = ", key)
            end
            
        else
            log.info("longLinkSocketTask.upload_tiandiren_rfid_info",">>>>>>>>> wait pub_packet_data_tiandiren timeout")
            --sys.wait(1000)
        end
    end
end)